BINDING_HEADER_BINDING_TEST = "Test bindings for Chapter 16"
BINDING_NAME_BINDING_TEST1 = "Test binding #1"
BINDING_NAME_BINDING_TEST2 = "Test binding #2"

local BUTTON_CAPTION = "Set Test Binding #%d"
local TOOLTIP_TEXT1 = "Left-click to add binding"
local TOOLTIP_TEXT2 = "Right-click to remove first binding"
local TOOLTIP_BINDING_LIST = "Currently bound: |cFFFFFFFF%s|r"
local LIST_SEPARATOR = "|r, |cFFFFFFFF"
local CAPTURE_TEXT = [[
Press a key or click this frame to set test binding #|cFFFFFFFF%d|r.

Press |cFFFFFFFFESC|r to cancel.]]

function BindingTest_Test1()
  print("Test binding #1 activated")
end

function BindingTest_Test2(keystate)
  if keystate == "down" then
    print("Test binding #2 pressed")
  else
    print("Test binding #2 released")
  end
end

function BindingTestButton_OnLoad(self)
  local id = self:GetID()
  self:SetText(BUTTON_CAPTION:format(id))
  self.action = "BINDING_TEST"..id
  self:RegisterForClicks("LeftButtonUp", "RightButtonUp")
end
function BindingTestButton_OnClick(self, button)
  if button == "LeftButton" then
    BindingTestCaptureFrame.button = self
    BindingTestCaptureFrame:Show()
  elseif button == "RightButton" then
    local key = GetBindingKey(self.action)
    if key then
      SetBinding(key, nil)
      SaveBindings(GetCurrentBindingSet())
    end
    BindingTestButton_OnEnter(self)
  end
end

function BindingTestButton_OnEnter(self)
  GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT")
  GameTooltip:AddLine(TOOLTIP_TEXT1)
  GameTooltip:AddLine(TOOLTIP_TEXT2)

  local list = {GetBindingKey(self.action)}
  for i, key in ipairs(list) do
    list[i] = GetBindingText(key, "KEY_")
  end

  GameTooltip:AddLine(
  TOOLTIP_BINDING_LIST:format(table.concat(list, LIST_SEPARATOR)),
  NORMAL_FONT_COLOR.r,
  NORMAL_FONT_COLOR.g,
  NORMAL_FONT_COLOR.b,
  1
  )

  GameTooltip:Show()
end

function BindingTestButton_OnLeave(self)
  GameTooltip:Hide()
end

function BindingTestCapture_OnLoad(self)
  self:RegisterForClicks("AnyUp")
end

function BindingTestCapture_OnShow(self)
  self.text:SetText(CAPTURE_TEXT:format(self.button:GetID()))
end

function BindingTestCapture_OnKeyDown(self, key)
  if key == "ESCAPE" then
    self:Hide()
    return
  end

  local modifier = key:sub(2)
  if  modifier == "SHIFT"
    or modifier == "CTRL"
    or modifier == "ALT"
    or key == "UNKNOWN" then
    return
  end

  if GetBindingFromClick(key) == "SCREENSHOT" then
    TakeScreenshot()
    return
  end

  if IsShiftKeyDown() then
    key = "SHIFT-"..key
  end
  if IsControlKeyDown() then
    key = "CTRL-"..key
  end
  if IsAltKeyDown() then
    key = "ALT-"..key
  end

  SetBinding(key, self.button.action)
  SaveBindings(GetCurrentBindingSet())
  self:Hide()
end

local buttonKeys = {
  ["MiddleButton"] = "BUTTON3",
  ["Button4"] = "BUTTON4",
  ["Button5"] = "BUTTON5"
}
function BindingTestCapture_OnClick(self, button)
  local key = buttonKeys[button]
  if not key then
    return
  end
  BindingTestCapture_OnKeyDown(self, key)
end
